#include <NaNs.mqh>

const bool _inTester = MQLInfoInteger(MQL_TESTER) && !MQLInfoInteger(MQL_VISUAL_MODE);

int OnCalculate(const int rates_total,
                const int prev_calculated,
                const int begin,
                const double &price[])
{
  ArraySetAsSeries(price, true);
  static bool surrogate = false; 
  // NB: _AppliedTo is not set correctly in tester, always equal 0!
  if(/*_AppliedTo >= 8 &&*/surrogate || !MathIsValidNumber(price[0]))
  {
    surrogate = true;
    // 51 bits (6+ bytes) are available for size, only 4 needed
    const int size = (int)(NaNs[price[0]] - NAN_QUIET); // element at the index size does not exist!
    static int prev;
    if(!prev_calculated) prev = 0;
    
    static int prevsize;
    if(size != prevsize)
    {
      // n = 5, '.' denotes valid data
      // 0 1 2 3 4 ->   as series
      // . 1              false
      // 4 3 2 1 0 <-   as series
      // . 3 = 5 - 1 - 1  true
      Print(iBars(_Symbol, _Period), " ", size, " ", iTime(_Symbol, _Period, iBars(_Symbol, _Period) - size), " ",
        DoubleToString(price[iBars(_Symbol, _Period) - size - 1 + 2], _Digits), " ",  // previous 1-st
        DoubleToString(price[iBars(_Symbol, _Period) - size - 1 + 1], _Digits));      // current 0-th
      prevsize = size;
    }
    
    if(!_inTester)
    {
      // output rightmost valid element
      Comment(TimeCurrent(), " ", iBars(_Symbol, _Period), " ", size, " ", iTime(_Symbol, _Period, iBars(_Symbol, _Period) - size));
    }
    
    ArraySetAsSeries(price, false); // restore default indexing
    prev = brbOnCalculate(size, prev, begin, price);
    return prev;
  }
  else
  {
    // Comment(price[10000000]); // throw an exception and force unload
  }
  
  ArraySetAsSeries(price, false); // restore default indexing
  return brbOnCalculate(rates_total, prev_calculated, begin, price);
}

#define OnCalculate brbOnCalculate
